@@ -36,10 +36,10 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
     {
         $rcmail = rcmail::get_instance();
 
-        self::$COMPOSE_ID = rcube_utils::get_input_value('_id', rcube_utils::INPUT_GET);
+        self::$COMPOSE_ID = rcube_utils::get_input_string('_id', rcube_utils::INPUT_GET);
         self::$COMPOSE    = null;
 
-        if (self::$COMPOSE_ID && $_SESSION['compose_data_' . self::$COMPOSE_ID]) {
+        if (self::$COMPOSE_ID && !empty($_SESSION['compose_data_' . self::$COMPOSE_ID])) {
             self::$COMPOSE =& $_SESSION['compose_data_' . self::$COMPOSE_ID];
         }
 
@@ -141,6 +141,7 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
 
         $compose_mode = null;
         $msg_uid      = null;
+        $options      = [];
 
         // get reference message and set compose mode
         if (!empty(self::$COMPOSE['param']['draft_uid'])) {
@@ -230,7 +231,7 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
                     self::$COMPOSE['reply_uid'] = self::$MESSAGE->context === null ? $msg_uid : null;
 
                     if (!empty(self::$COMPOSE['param']['all'])) {
-                        self::$MESSAGE->reply_all = self::$COMPOSE['param']['all'];
+                        self::$COMPOSE['reply_all'] = self::$COMPOSE['param']['all'];
                     }
                 }
                 else {
@@ -255,19 +256,25 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
                         // get reply_uid/forward_uid to flag the original message when sending
                         $info = rcmail_sendmail::draftinfo_decode($draft_info);
 
-                        if ($info['type'] == 'reply') {
-                            self::$COMPOSE['reply_uid'] = $info['uid'];
-                        }
-                        else if ($info['type'] == 'forward') {
-                            self::$COMPOSE['forward_uid'] = $info['uid'];
+                        if (!empty($info['type'])) {
+                            if ($info['type'] == 'reply') {
+                                self::$COMPOSE['reply_uid'] = $info['uid'];
+                            }
+                            else if ($info['type'] == 'forward') {
+                                self::$COMPOSE['forward_uid'] = $info['uid'];
+                            }
                         }
 
-                        self::$COMPOSE['mailbox'] = $info['folder'];
+                        if (!empty($info['dsn']) && $info['dsn'] === 'on') {
+                            $options['dsn_enabled'] = true;
+                        }
+
+                        self::$COMPOSE['mailbox'] = $info['folder'] ?? null;
 
                         // Save the sent message in the same folder of the message being replied to
                         if (
                             $rcmail->config->get('reply_same_folder')
-                            && ($sent_folder = $info['folder'])
+                            && ($sent_folder = self::$COMPOSE['mailbox'])
                             && rcmail_sendmail::check_sent_folder($sent_folder, false)
                         ) {
                             self::$COMPOSE['param']['sent_mbox'] = $sent_folder;
@@ -306,8 +313,10 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
             $rcmail->output->set_env('reply_msgid', self::$COMPOSE['reply_msgid']);
         }
 
+        $options['message'] = self::$MESSAGE;
+
         // Initialize helper class to build the UI
-        self::$SENDMAIL = new rcmail_sendmail(self::$COMPOSE, ['message' => self::$MESSAGE]);
+        self::$SENDMAIL = new rcmail_sendmail(self::$COMPOSE, $options);
 
         // process self::$MESSAGE body/attachments, set self::$MESSAGE_BODY/$HTML_MODE vars and some session data
         self::$MESSAGE_BODY = self::prepare_message_body();
@@ -555,8 +564,8 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
 
         // use posted message body
         if (!empty($_POST['_message'])) {
-            $body   = rcube_utils::get_input_value('_message', rcube_utils::INPUT_POST, true);
-            $isHtml = (bool) rcube_utils::get_input_value('_is_html', rcube_utils::INPUT_POST);
+            $body   = rcube_utils::get_input_string('_message', rcube_utils::INPUT_POST, true);
+            $isHtml = (bool) rcube_utils::get_input_string('_is_html', rcube_utils::INPUT_POST);
         }
         else if (!empty(self::$COMPOSE['param']['body'])) {
             $body   = self::$COMPOSE['param']['body'];
@@ -628,9 +637,6 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
                     }
                 }
             }
-            else {
-                $body = self::compose_part_body(self::$MESSAGE, $isHtml);
-            }
 
             // compose reply-body
             if (self::$COMPOSE['mode'] == rcmail_sendmail::MODE_REPLY) {
@@ -689,8 +695,20 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
         return $body;
     }
 
+    /**
+     * Prepare message part body for composition
+     *
+     * @param rcube_message_part $part   Message part
+     * @param bool               $isHtml Use HTML mode
+     *
+     * @return string Message body text
+     */
     public static function compose_part_body($part, $isHtml = false)
     {
+        if (!$part instanceof rcube_message_part) {
+            return '';
+        }
+
         // Check if we have enough memory to handle the message in it
         // #1487424: we need up to 10x more memory than the body
         if (!rcube_utils::mem_check($part->size * 10)) {
@@ -783,7 +801,8 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
         // If desired, set this textarea to be editable by TinyMCE
         $attrib['data-html-editor'] = true;
         if (self::$HTML_MODE) {
-            $attrib['class'] = trim((isset($attrib['class']) ? $attrib['class'] : '') . ' mce_editor');
+            $attrib['class'] = trim(($attrib['class'] ?? '') . ' mce_editor');
+            $attrib['data-html-editor-content-element'] = $attrib['id'] . '-content';
         }
 
         $attrib['name'] = '_message';
@@ -799,12 +818,22 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
 
         $rcmail->output->set_env('composebody', $attrib['id']);
 
+        $content = $hidden->show() . "\n";
+
+        // We're adding a hidden textarea with the HTML content to workaround browsers' performance
+        // issues with rendering/loading long content. It will be copied to the main editor (#8108)
+        if (self::$HTML_MODE && strlen(self::$MESSAGE_BODY) > 50 * 1024) {
+            $contentArea = new html_textarea(['style' => 'display:none', 'id' => $attrib['id'] . '-content']);
+            $content .= $contentArea->show(self::$MESSAGE_BODY) . "\n" . $textarea->show();
+        }
+        else {
+            $content .= $textarea->show(self::$MESSAGE_BODY);
+        }
+
         // include HTML editor
         self::html_editor();
 
-        return ($form_start ? "$form_start\n" : '')
-            . "\n" . $hidden->show() . "\n" . $textarea->show(self::$MESSAGE_BODY)
-            . ($form_end ? "\n$form_end\n" : '');
+        return "$form_start\n$content\n$form_end\n";
     }
 
     public static function create_reply_body($body, $bodyIsHtml)
@@ -828,9 +857,10 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
         }
 
         if (!$bodyIsHtml) {
-            // soft-wrap and quote message text
-            $line_length = $rcmail->config->get('line_length', 72);
-            $body = self::wrap_and_quote($body, $line_length, $reply_indent);
+            // quote the message text
+            if ($reply_indent) {
+                $body = self::quote_text($body);
+            }
 
             if ($reply_mode > 0) { // top-posting
                 $prefix = "\n\n\n" . $prefix;
@@ -1072,7 +1102,7 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
                         continue;
                     }
 
-                    $idx = $part->content_id ? ('cid:' . $part->content_id) : $part->content_location;
+                    $idx = $part->content_id ? ('cid:' . $part->content_id) : $part->content_location ?? null;
 
                     if ($idx && isset(self::$CID_MAP[$idx]) && strpos($message_body, self::$CID_MAP[$idx]) !== false) {
                         $replace = self::$CID_MAP[$idx];
@@ -1350,7 +1380,7 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
         if (!empty($attrib['icon_pos']) && $attrib['icon_pos'] == 'left') {
             self::$COMPOSE['icon_pos'] = 'left';
         }
-        $icon_pos = isset(self::$COMPOSE['icon_pos']) ? self::$COMPOSE['icon_pos'] : null;
+        $icon_pos = self::$COMPOSE['icon_pos'] ?? null;
 
         if (!empty(self::$COMPOSE['attachments'])) {
             if (!empty($attrib['deleteicon'])) {
@@ -1563,30 +1593,34 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
 
     /**
      * Responses list object for templates
+     *
+     * @param array $attrib Object attributes
+     *
+     * @return string HTML content
      */
     public static function compose_responses_list($attrib)
     {
         $rcmail = rcmail::get_instance();
 
-        $attrib += ['id' => 'rcmresponseslist', 'tagname' => 'ul', 'cols' => 1];
+        $attrib += ['id' => 'rcmresponseslist', 'tagname' => 'ul', 'cols' => 1, 'itemclass' => ''];
 
-        $jsenv = [];
         $list = new html_table($attrib);
 
-        foreach ($rcmail->get_compose_responses(true) as $response) {
-            $key  = $response['key'];
+        foreach ($rcmail->get_compose_responses() as $response) {
             $item = html::a([
-                    'href'         => '#' . urlencode($response['name']),
+                    'href'         => '#response-' . urlencode($response['id']),
                     'class'        => rtrim('insertresponse ' . $attrib['itemclass']),
                     'unselectable' => 'on',
                     'tabindex'     => '0',
-                    'rel'          => $key,
+                    'onclick'      => sprintf(
+                        "return %s.command('insert-response', '%s', this, event)",
+                        rcmail_output::JS_OBJECT_NAME,
+                        rcube::JQ($response['id'])
+                    ),
                 ],
                 rcube::Q($response['name'])
             );
 
-            $jsenv[$key] = $response;
-
             $list->add([], $item);
         }
 
@@ -1603,8 +1637,6 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
             ));
         }
 
-        // set client env
-        $rcmail->output->set_env('textresponses', $jsenv);
         $rcmail->output->add_gui_object('responseslist', $attrib['id']);
 
         return $list->show();
@@ -1624,7 +1656,7 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
         }
         else if ($message instanceof rcube_message) {
             // the whole message requested
-            $size     = isset($message->size) ? $message->size : null;
+            $size     = $message->size ?? null;
             $mimetype = 'message/rfc822';
             $filename = !empty($params['filename']) ? $params['filename'] : 'message_rfc822.eml';
         }
@@ -1679,9 +1711,9 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
             'mimetype'   => $mimetype,
             'content_id' => !empty($part) && isset($part->content_id) ? $part->content_id : null,
             'data'       => $data,
-            'path'       => isset($path) ? $path : null,
+            'path'       => $path ?? null,
             'size'       => isset($path) ? filesize($path) : strlen($data),
-            'charset'    => !empty($part) ? $part->charset : (isset($params['charset']) ? $params['charset'] : null),
+            'charset'    => !empty($part) ? $part->charset : ($params['charset'] ?? null),
         ];
 
         $attachment = $rcmail->plugins->exec_hook('attachment_save', $attachment);
@@ -1711,4 +1743,24 @@ class rcmail_action_mail_compose extends rcmail_action_mail_index
 
         return false;
     }
+
+    /**
+     * Add quotation (>) to a replied message text.
+     *
+     * @param string $text Text to quote
+     *
+     * @return string The quoted text
+     */
+    public static function quote_text($text)
+    {
+        $lines = preg_split('/\r?\n/', trim($text));
+        $out   = '';
+
+        foreach ($lines as $line) {
+            $quoted = isset($line[0]) && $line[0] == '>';
+            $out .= '>' . ($quoted ? '' : ' ') . $line . "\n";
+        }
+
+        return rtrim($out, "\n");
+    }
 }
