@@ -541,7 +541,7 @@ class rcube_ldap extends rcube_addressbook
     /**
      * Set internal list page
      *
-     * @param  number  Page number to list
+     * @param number $page Page number to list
      */
     function set_page($page)
     {
@@ -552,7 +552,7 @@ class rcube_ldap extends rcube_addressbook
     /**
      * Set internal page size
      *
-     * @param  number  Number of records to display on one page
+     * @param number $size Number of records to display on one page
      */
     function set_pagesize($size)
     {
@@ -914,20 +914,18 @@ class rcube_ldap extends rcube_addressbook
             $attributes = [];
 
             if ($fields == '*') {
-                $attributes = (array) ($this->prop['search_fields'] ?? []);
+                $fields = (array) ($this->prop['search_fields'] ?? []);
 
                 // If search fields aren't configured use some common fields
-                if (empty($search_fields)) {
+                if (empty($fields)) {
                     $fields = ['name', 'surname', 'firstname', 'email'];
                 }
             }
 
             // map address book fields into ldap attributes
-            if (empty($attributes)) {
-                foreach ((array) $fields as $field) {
-                    if (!empty($this->coltypes[$field]) && !empty($this->coltypes[$field]['attributes'])) {
-                        $attributes = array_merge($attributes, (array) $this->coltypes[$field]['attributes']);
-                    }
+            foreach ((array) $fields as $field) {
+                if (!empty($this->coltypes[$field]) && !empty($this->coltypes[$field]['attributes'])) {
+                    $attributes = array_merge($attributes, (array) $this->coltypes[$field]['attributes']);
                 }
             }
 
@@ -1917,12 +1915,12 @@ class rcube_ldap extends rcube_addressbook
         }
 
         $base_dn    = $this->groups_base_dn;
-        $filter     = $this->prop['groups']['filter'];
+        $filter     = $this->prop['groups']['filter'] ?? null;
         $scope      = $this->prop['groups']['scope'] ?? 'sub';
         $name_attr  = !empty($this->prop['groups']['name_attr']) ? $this->prop['groups']['name_attr'] : 'cn';
         $email_attr = !empty($this->prop['groups']['email_attr']) ? $this->prop['groups']['email_attr'] : 'mail';
         $sort_attrs = (array) (!empty($this->prop['groups']['sort']) ? $this->prop['groups']['sort'] : $name_attr);
-        $sort_attr  = $sort_attrs[0];
+        $sort_attr  = array_first($sort_attrs) ?: $name_attr;
         $page_size  = 200;
 
         $ldap = $this->ldap;
@@ -1977,15 +1975,15 @@ class rcube_ldap extends rcube_addressbook
             $groups[$group_id]['ID'] = $group_id;
             $groups[$group_id]['dn'] = $entry['dn'];
             $groups[$group_id]['name'] = $group_name;
-            $groups[$group_id]['member_attr'] = $this->get_group_member_attr($entry['objectclass']);
+            $groups[$group_id]['member_attr'] = $this->get_group_member_attr($entry['objectclass'] ?? null);
 
             // list email attributes of a group
-            for ($j=0; $entry[$email_attr] && $j < $entry[$email_attr]['count']; $j++) {
+            for ($j=0; !empty($entry[$email_attr]) && $j < $entry[$email_attr]['count']; $j++) {
                 if (strpos($entry[$email_attr][$j], '@') > 0)
                     $groups[$group_id]['email'][] = $entry[$email_attr][$j];
             }
 
-            $group_sortnames[] = mb_strtolower($entry[$sort_attr][0]);
+            $group_sortnames[] = mb_strtolower($entry[$sort_attr][0] ?? '');
         }
 
         // recursive call can exit here
@@ -2272,7 +2270,7 @@ class rcube_ldap extends rcube_addressbook
      */
     private function get_group_member_attr($object_classes = [], $default = 'member')
     {
-        if (empty($object_classes)) {
+        if (empty($object_classes) && !empty($this->prop['groups']['object_classes'])) {
             $object_classes = $this->prop['groups']['object_classes'];
         }
 
